
import 'dart:async';
import 'dart:convert';
import 'dart:html';
import 'package:webview/webview.dart';

main() => scheduleMicrotask(onLoad);

var isLoading = false;
var webview;

onLoad() {
  webview = new Webview.wrap(document.querySelector('webview'));
  
  document.querySelector('#back').onClick.listen((e) => webview.back());  
  document.querySelector('#forward').onClick.listen((e) => webview.forward());
  
  document.querySelector('#home').onClick.listen((e) => 
      navigateTo('http://www.google.com/'));
  
  document.querySelector('#reload').onClick.listen((e) {
    if (isLoading) {
      webview.stop();
    } else {
      webview.reload();
    }
  });
  
  document.querySelector('#reload').on['webkitAnimationIteration'].listen((e) {
    if (!isLoading) {
      document.body.classes.remove('loading');
    }
  });
  
  document.querySelector('#terminate').onClick.listen((e) => 
      webview.terminate());
    
  document.querySelector('#location-form').onSubmit.listen((e) {
    e.preventDefault();
    navigateTo((document.querySelector('#location') as InputElement).value);
  });
  
  webview.onExit.listen(handleExit);
  webview.onLoadAbort.listen(handleLoadAbort);
  webview.onLoadCommit.listen(handleLoadCommit);
  webview.onLoadRedirect.listen(handleLoadRedirect);
  webview.onLoadStart.listen(handleLoadStart);
  webview.onLoadStop.listen(handleLoadStop);
}

handleExit(event) {
  var detail = JSON.decode(event.detail);
  document.body.classes.add('exited');
  if (detail['reason'] == 'abnormal') {
    document.body.classes.add('crashed');
  } else if (detail['reason'] == 'killed') {
    document.body.classes.add('killed');
  }
}

handleLoadAbort(event) { }

handleLoadCommit(event) {
  var detail = JSON.decode(event.detail);

  resetExitedState();
  
  if (!detail['isTopLevel']) return;

  (document.querySelector('#location') as InputElement).value = detail['url'];

  (document.querySelector('#back') as ButtonElement).disabled = 
      !webview.canGoBack;
  (document.querySelector('#forward') as ButtonElement).disabled = 
      !webview.canGoForward;
}

handleLoadRedirect(event) {
  var detail = JSON.decode(event.detail);

  resetExitedState();
  
  if (!detail['isTopLevel']) return;

  (document.querySelector('#location') as InputElement).value = 
      detail['newUrl'];
}

handleLoadStart(event) {
  var detail = JSON.decode(event.detail);
  document.body.classes.add('loading');
  isLoading = true;  
  resetExitedState();
  
  if (!detail['isTopLevel']) return;

  (document.querySelector('#location') as InputElement).value = detail['url'];
}

handleLoadStop(event) {  
  // We don't remove the loading class immediately, instead we let the animation
  // finish, so that the spinner doesn't jerkily reset back to the 0 position.
  isLoading = false;
}

navigateTo(url) {
  resetExitedState();
  webview.src = url;
}

resetExitedState() {
  document.body.classes.remove('exited');
  document.body.classes.remove('crashed');
  document.body.classes.remove('killed');
}
